package pwhash

import "testing"

func TestArgon2(t *testing.T) {
	testImpl(t, &Argon2PasswordHash{})
}

func TestScrypt(t *testing.T) {
	testImpl(t, &ScryptPasswordHash{})
}

func TestSystemCrypt(t *testing.T) {
	testImpl(t, &SystemCryptPasswordHash{})
}

func TestDefault(t *testing.T) {
	testImpl(t, nil)
}

func testImpl(t *testing.T, h PasswordHash) {
	pw1 := "password 1"
	pw2 := "password 2"

	var enc1, enc2 string
	if h == nil {
		enc1 = Encrypt(pw1)
		enc2 = Encrypt(pw2)
	} else {
		enc1 = h.Encrypt(pw1)
		enc2 = h.Encrypt(pw2)
	}
	//t.Logf("enc1=%s", enc1)

	testData := []struct {
		enc            string
		pw             string
		expectedResult bool
	}{
		{enc1, pw1, true},
		{enc2, pw2, true},
		{enc1, pw2, false},
		{enc2, pw1, false},
		{enc1, "", false},
		{enc1, "foo", false},
	}

	for _, td := range testData {
		var result bool
		if h == nil {
			result = ComparePassword(td.enc, td.pw)
		} else {
			result = h.ComparePassword(td.enc, td.pw)
		}
		if result != td.expectedResult {
			t.Errorf("compare(%s, %s): got %v, expected %v", td.enc, td.pw, result, td.expectedResult)
		}
	}
}
